## Wie funktioniert die Umwandlung?

Kern der Umwandlung ist die Klasse `MDMetadata2Dataset` und darin die Methode `convert(org.dom4j.Element)`. Sie erwartet ein `MD_Metadata` Element, aus dem eine Resource vom Typ `dcat:Dataset` (und verbundene `dcat:Distribution`) erzeugt wird.

Neben den Metadaten, die im `MD_Metadata` Element selbst enthalten sind, wird über die Klasse `CswInterface` eine Suche nach gekoppelten Services durchgeführt (Methodde `CswInterface#findOperatesOn`).

### Dienste

Die Umwandlung erfolgt noch nach DCAT-AP 1 Logik. Somit wird ein Dienst (service) nur dann umgewandelt, wenn er mit einem Datensatz gekoppelt ist (Daten-Dienste-Kopplung). Der Dienst wird dann zu einer *Distribution* des entsprechenden Datensatzes.

### Kontaktdaten

Für die Umwandlung von Kontaktdaten sind zwei Methoden zuständig: `createFoafOrg`, die eine Resource vom Typ `foaf:Organization` erzeugt und `createVcardOrganization`, die eine Resource vom Typ `vcard:Organization` erzeugt. 

Siehe auch Datei [Kontaktdaten.md](Kontaktdaten.md).

### DCAT-Kategorien

Zusätzlich zu dem Mapping auf Basis der `gmd:topicCategory` werden zusätzlich einige Schlüsselwörter ausgewertet, um die DCAT-Kategorie zu bestimmen. Diese Zuordnung ist in der Datei `src/main/resources/keyword-mapping.json` zu finden.

### Koordinatenreferenzsystem

Informationen zu Koordinatenreferenzsystemen in `gmd:referenceSystemInfo` werden auf passende `dct:conformsTo`-Angaben abgebildet. 

### Angaben zur Konformität

Angaben zur Konformität eines Datensatzes, wie z.B. *VERORDNUNG (EG) Nr. 1089/2010 DER KOMMISSION vom 23. November 2010 zur Durchführung der Richtlinie 2007/2/EG des Europäischen Parlaments und des Rates hinsichtlich der Interoperabilität von Geodatensätzen und -diensten* werden auf passende `dct:conformsTo`-Angaben abgebildet.

### dcatap:applicableLegislation

Ist ein Datensatz mit dem Schlüsselwort `INSPIRE`, `inspire` oder `inspireidentifiziert` markiert, wird eine Kennzeichnung gemäß <https://semiceu.github.io/uri.semic.eu-generated/DCAT-AP/releases/2.2.0-hvd/> vorgenommen: 

```ttl
   dcatap:applicableLegislation <http://data.europa.eu/eli/dir/2007/2/2019-06-26>
```

### Bestimmen des Formats

Da in den ISO-Metadaten das Format üblicherweise nicht pro *transfer option* angegeben ist, wird auf verschiedenen Wegen versucht, das Format zu bestimmen. 

Zunächst wird nachgesehen, ob ein Element `gmd:applicationProfile` vorhanden ist und einen vom EU Publications Office vorgegebeben URI für das Format enthält. Dieser würde verwendet.

Wenn der URL (ohne Berücksichtigung von Klein- und Großschreibung) `service=wms` und der URL keinen Parameter `outputformat` enhält, wird als Format `WMS_SRVC` ermittelt. Ist ein Parameter `outputformat` wird eine Info-Meldung geloggt und kein Format ermittelt.

Wenn er URL (ohne Berücksichtigung von Klein- und Großschreibung) `service=wfs` enthält, wird auf den Parameter `outputformat` geschaut:

- ohne → `WFS_SRVC`
- `outputformat=shape-zip` → `SHP`
- `outputformat=shape/zip` → `SHP`
- `outputformat=csv` → `CSV`
- bei einem sonstigen Wert wird eine Info-Meldung geloggt und kein Format ermittelt.

Konnte auf diese Weise kein Wert ermittelt werden, wird der Namen bzw. die Beschreibung (sofern der Name leer oder identisch mit dem URL ist) der *transfer option* geprüft, ob sie bestimmte Werte hat:

- `Feedlink des Dienstes` → `ATOM`
- `Feed Link des Dienstes` → `ATOM`
- `AtomFeed Viewer` → `HTML`
