package de.landsh.opendata.csw2dcat;

import de.landsh.opendata.csw2dcat.vocabulary.*;
import org.apache.jena.rdf.model.Model;
import org.apache.jena.rdf.model.ModelFactory;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.vocabulary.*;
import org.dom4j.DocumentException;
import org.dom4j.Element;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.StringWriter;
import java.nio.charset.StandardCharsets;
import java.util.Collections;

public class CommandLine {
    private static final Logger log = LoggerFactory.getLogger(CommandLine.class);

    static void setNsPrefixes(Model model) {
        model.setNsPrefix("dcat", DCAT.NS);
        model.setNsPrefix("dct", DCTerms.NS);
        model.setNsPrefix("vcard", VCARD4.NS);
        model.setNsPrefix("dcatde", DCATAPde.NS);
        model.setNsPrefix("adms", Adms.NS);
        model.setNsPrefix("cnt", Content.NS);
        model.setNsPrefix("geosparql", "http://www.opengis.net/ont/geosparql#");
        model.setNsPrefix("locn", Locn.NS);
        model.setNsPrefix("foaf", FOAF.NS);
        model.setNsPrefix("hydra", Hydra.NS);
        model.setNsPrefix("schema", SchemaOrg.NS);
    }

    public static void main(String[] args) throws IOException, DocumentException {
        if (args.length == 0) {
            System.err.println("USAGE: Csw2Dcat <BASE_URL_OF_WEB_FEATURE_SERVICE>");
            System.exit(1);
        }

        String catalog = CommandLine.convertCatalog(args[0]);
        FileOutputStream out = new FileOutputStream("/tmp/catalog.rdf");
        out.write(catalog.getBytes(StandardCharsets.UTF_8));
        out.close();
    }
    public static String convertCatalog(String url) throws DocumentException, IOException {
        final Model model = ModelFactory.createDefaultModel();
        final CswInterface cswInterface = new CswInterface(url);

        final BridgeSettings settings = new BridgeSettings();
        settings.baseIRI   = "http://registry.gdi-de.org/id/de.sh/";
        settings.contributorId = "http://example.org";

        final MDMetadata2Dataset mdMetadata2Dataset = new MDMetadata2Dataset(model, cswInterface, settings);

        final Resource catalog = model.createResource();
        catalog.addProperty(RDF.type, DCAT.Catalog);



        int startPosition = 1;
        int count = 0;
        while (startPosition == 1 || count == cswInterface.getRecordsPerPage()) {
            count = 0;
            log.info("Getting records from CSW starting at position {}", startPosition);
            final CswInterface.GetRecordsStatus status = new CswInterface.GetRecordsStatus();
            for (Element element : cswInterface.getRecords(startPosition, status, Collections.emptyMap())) {
                final Resource dataset = mdMetadata2Dataset.convert(element);
                if (dataset != null) {
                    catalog.addProperty(DCAT.dataset, dataset);
                }
                count++;
            }
            log.info("Received {} records", count);
            if (count == 0) {
                break;
            }
            startPosition = startPosition + count;
        }


        setNsPrefixes(model);

        StringWriter out = new StringWriter();
        model.write(out, "RDF/XML-ABBREV");
        out.close();

        return out.toString();
    }
}
