package de.landsh.opendata.csw2dcat;

import org.apache.commons.lang3.StringUtils;

import java.util.regex.Pattern;

/**
 * Helper class with a static method to clean telephone numbers.
 */
public class TelephoneNumberClean {
    public static final Pattern tel = Pattern.compile("^tel:\\+(?:[0-9] ?){6,14}[0-9]$");

    static String numberToURI(String input) {

        String phoneNumber = StringUtils.trimToNull(input);
        if (phoneNumber == null) {
            return null;
        }

        // input is already in the expected format
        if (tel.matcher(input).matches()) {
            return input;
        }

        // remove brackets
        phoneNumber = phoneNumber.replaceAll("[()]", "");

        // replace whitespaces and slashes with dashes
        phoneNumber = phoneNumber.replaceAll("[ /]", "-");

        // At this point, the phone number may only consist of numbers and dashes (and an optional leading plus sign).
        if (!phoneNumber.replaceFirst("^\\+", "").replaceAll("[0-9-]", "").isEmpty()) {
            throw new IllegalArgumentException(input + " is not a phone number.");
        }

        if (!phoneNumber.startsWith("+")) {
            // we assume a German telephone number
            if (phoneNumber.startsWith("0")) {
                phoneNumber = "+49-" + phoneNumber.replaceAll("^0+", "");
            } else {
                phoneNumber = "+49-" + phoneNumber;
            }
        }

        // the input might have contained several zeros after the country prefix
        phoneNumber = phoneNumber.replaceFirst("^\\+490+", "+49");
        phoneNumber = phoneNumber.replaceFirst("^\\+49(-0)+", "+49-");

        return "tel:" + phoneNumber;
    }
}
