package de.landsh.opendata.csw2dcat;

import org.apache.commons.lang3.StringUtils;
import org.dom4j.*;
import org.dom4j.io.SAXReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.FileWriter;
import java.io.IOException;
import java.util.SortedSet;
import java.util.TreeSet;

/**
 * This tool downloads lists from the EU publication office (allowed fiel types) and stores them as local files.
 */
public class UpdateLists {

    private static final Logger log = LoggerFactory.getLogger(UpdateLists.class);

    private static final QName RDF_DESCRIPTION = new QName("Description", Namespace.get("http://www.w3.org/1999/02/22-rdf-syntax-ns#"));
    private static final QName RDF_ABOUT = new QName("about", Namespace.get("http://www.w3.org/1999/02/22-rdf-syntax-ns#"));

    public static void main(String[] args) throws Exception {
        new UpdateLists().loadFormats();
    }

    void loadFormats() throws IOException, DocumentException {

        SortedSet<String> formats = new TreeSet();
        SortedSet<String> services = new TreeSet();

        SAXReader saxReader = new SAXReader();

        log.info("Loading list for allowed file types from EU publications office...");
        Document doc = saxReader.read("https://publications.europa.eu/resource/authority/file-type");

        for (Element element : doc.getRootElement().elements(RDF_DESCRIPTION)) {
            final String uri = element.attributeValue(RDF_ABOUT);
            if (StringUtils.startsWith(uri, "http://publications.europa.eu/resource/authority/file-type/")) {
                final String format = StringUtils.substringAfter(uri, "http://publications.europa.eu/resource/authority/file-type/");
                if (format.endsWith("_SRVC")) {
                    services.add(StringUtils.substringBefore(format, "_SRVC"));
                } else {
                    formats.add(format);
                }
            }
        }

        try (FileWriter writer = new FileWriter("src/main/resources/allowed-formats.txt")) {
            for (String s : formats) {
                writer.write(s);
                writer.write('\n');
            }
            log.info("Wrote file src/main/resources/allowed-formats.txt.");
        }
        try (FileWriter writer = new FileWriter("src/main/resources/allowed-services.txt")) {
            for (String s : services) {
                writer.write(s);
                writer.write('\n');
            }
            log.info("Wrote file src/main/resources/allowed-services.txt.");
        }
    }

}
