package de.landsh.opendata.csw2dcat;

import org.apache.jena.rdf.model.Model;
import org.apache.jena.rdf.model.Resource;
import org.apache.jena.rdf.model.ResourceFactory;
import org.apache.jena.vocabulary.DCAT;
import org.apache.jena.vocabulary.DCTerms;
import org.apache.jena.vocabulary.VCARD4;
import org.dom4j.Document;
import org.dom4j.io.SAXReader;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

class GenericMetadataConverterTest {
    public static final String NS_GMD = "http://www.isotc211.org/2005/gmd#";
    private final SAXReader saxReader = new SAXReader();
    private final GenericMetadataConverter converter = new GenericMetadataConverter();

    private static void setNsPrefixes(Model model) {
        model.setNsPrefix("dcat", DCAT.NS);
        model.setNsPrefix("dct", DCTerms.NS);
        model.setNsPrefix("vcard", VCARD4.NS);
        model.setNsPrefix("dcatde", "http://dcat-ap.de/def/dcatde/");
        model.setNsPrefix("adms", "http://www.w3.org/ns/adms#");
        model.setNsPrefix("cnt", "http://www.w3.org/2011/content#");
        model.setNsPrefix("geosparql", "http://www.opengis.net/ont/geosparql#");
        model.setNsPrefix("locn", "http://www.w3.org/ns/locn#");
        model.setNsPrefix("foaf", "http://xmlns.com/foaf/0.1/");
        model.setNsPrefix("gco", "http://www.isotc211.org/2005/gco");
        model.setNsPrefix("gmd", NS_GMD);
        model.setNsPrefix("gmx", "http://www.isotc211.org/2005/gmx#");
        model.setNsPrefix("srv", "http://www.isotc211.org/2005/srv#");
    }

    @Test
    void iso2dcat() throws Exception {
        final Document inputDocument = saxReader.read(getClass().getResourceAsStream("/9066e59a-c4e5-4254-b3e1-eb4eb16bdc53.xml"));

        Resource dataset = converter.iso2dcat(inputDocument.getRootElement());

        // setNsPrefixes(dataset.getModel());
        // dataset.getModel().write(System.out, "RDF/XML-ABBREV");
        // System.out.flush();

        assertTrue(dataset.hasProperty(ResourceFactory.createProperty(NS_GMD, "fileIdentifier"), "9066e59a-c4e5-4254-b3e1-eb4eb16bdc53"));

        assertEquals(ResourceFactory.createResource("http://standards.iso.org/iso/19139/resources/gmxCodelists.xml#MD_ScopeCode_service"),
                dataset.getPropertyResourceValue(ResourceFactory.createProperty(NS_GMD, "hierarchyLevel")));
        assertEquals(ResourceFactory.createResource("http://standards.iso.org/iso/19139/resources/gmxCodelists.xml#MD_CharacterSetCode_utf8"),
                dataset.getPropertyResourceValue(ResourceFactory.createProperty(NS_GMD, "characterSet")));
    }
}
