package de.landsh.opendata.csw2dcat;

import org.apache.jena.datatypes.xsd.XSDDatatype;
import org.apache.jena.rdf.model.*;
import org.apache.jena.vocabulary.DCAT;
import org.apache.jena.vocabulary.DCTerms;
import org.apache.jena.vocabulary.RDF;
import org.junit.jupiter.api.Test;

import java.util.Objects;

import static org.junit.jupiter.api.Assertions.fail;

public class RdfTester {

    /**
     * Checks that the <code>resourceToTest</code> contains the specified <code>property</code> with the same values
     * as in the <code>reference</code> model.
     */
    static void assertStatementPresent(Resource resourceToTest, Property property, Resource reference) {
        // Loop through the statements of the reference and check if the statement is present.
        final StmtIterator itReference = reference.listProperties(property);
        while (itReference.hasNext()) {
            final Statement referenceStatement = itReference.next();

            boolean found = false;
            if (referenceStatement.getObject().isLiteral()) {
                final Literal referenceLiteral = referenceStatement.getLiteral();

                final StmtIterator it = resourceToTest.listProperties(property);
                while (it.hasNext()) {
                    final Literal foundLiteral = it.next().getLiteral();

                    found |= Objects.equals(foundLiteral.getLexicalForm(), referenceLiteral.getLexicalForm())
                            && Objects.equals(foundLiteral.getDatatypeURI(), referenceLiteral.getDatatypeURI())
                            && Objects.equals(foundLiteral.getLanguage(), referenceLiteral.getLanguage());
                }
            } else if( referenceStatement.getObject().isURIResource()) {
                found = resourceToTest.hasProperty(property, referenceStatement.getObject());
            }else {
                // The object is a complex Resource -> check recursively
                final StmtIterator it = resourceToTest.listProperties(property);
                while (it.hasNext()) {
                    final Resource resource = (Resource) it.next().getObject();
                    final Resource referenceResource = (Resource) referenceStatement.getObject();
                    assertStatementPresent(resource , property, referenceResource);
                    found = true;
                }

            }
            if (!found) {
                fail("Statement " + referenceStatement + " not found.");
            }
        }

    }

    /**
     * Returns one resource with the specified type from the model. Fail if the model contains more than one
     * resource with the specified type.
     */
    static Resource getOneResource(Model model, Resource type) {
        ResIterator it = model.listResourcesWithProperty(RDF.type, type);
        Resource result = it.next();

        if (it.hasNext()) {
            fail("More than one resource of type " + type);
        }

        return result;
    }

}
