package de.landsh.opendata.csw2dcat;

import org.apache.jena.datatypes.xsd.XSDDatatype;
import org.apache.jena.rdf.model.*;
import org.apache.jena.vocabulary.DCAT;
import org.apache.jena.vocabulary.DCTerms;
import org.apache.jena.vocabulary.RDF;
import org.junit.jupiter.api.Test;

import java.util.Objects;

import static org.junit.jupiter.api.Assertions.fail;
import static de.landsh.opendata.csw2dcat.RdfTester.assertStatementPresent;
import static de.landsh.opendata.csw2dcat.RdfTester.getOneResource;

public class RdfTesterTests {


    @Test
    public void testAssertStatementPresent_sameResource() {
        Model model = ModelFactory.createDefaultModel();
        model.read(getClass().getResourceAsStream("/f7f90143-c2ad-46b2-934d-93dfd5e0f031.rdf"), null);

        Model model2 = ModelFactory.createDefaultModel();
        model2.read(getClass().getResourceAsStream("/f7f90143-c2ad-46b2-934d-93dfd5e0f031.rdf"), null);

        Resource reference = getOneResource(model2, DCAT.Dataset);
        Resource dataset = getOneResource(model, DCAT.Dataset);

        assertStatementPresent(dataset, DCTerms.title, reference);
        assertStatementPresent(dataset, DCAT.keyword, reference);
        assertStatementPresent(dataset, DCTerms.identifier, reference);
        assertStatementPresent(dataset,DCTerms.accessRights, reference);
        assertStatementPresent(dataset, DCTerms.spatial, reference);

    }

    @Test
    public void testAssertStatementPresent_generated() {
        Model model = ModelFactory.createDefaultModel();
        model.read(getClass().getResourceAsStream("/f7f90143-c2ad-46b2-934d-93dfd5e0f031.rdf"), null);

        Resource reference = getOneResource(model, DCAT.Dataset);

        Resource resource = model.createResource();
        resource.addLiteral(DCTerms.title, model.createLiteral("Straßenschlüsselverzeichnis der Stadt Elmshorn ohne Koordinatenbezug", "de"));
        resource.addLiteral(DCAT.keyword, model.createLiteral("inspireidentifiziert", "de"));
        resource.addLiteral(DCAT.keyword, model.createLiteral("opendata", "de"));
        resource.addLiteral(DCAT.keyword, model.createLiteral("Stadt Elmshorn", "de"));
        resource.addLiteral(DCAT.keyword, model.createLiteral("Elmshorn", "de"));
        resource.addLiteral(DCAT.keyword, model.createLiteral("Straßenschlüssel", "de"));
        resource.addLiteral(DCAT.keyword, model.createLiteral("Straßennamen", "de"));
        resource.addLiteral(DCTerms.identifier, model.createTypedLiteral("f7f90143-c2ad-46b2-934d-93dfd5e0f031", XSDDatatype.XSDstring));
        resource.addLiteral(DCTerms.accessRights, model.createResource("http://inspire.ec.europa.eu/metadata-codelist/LimitationsOnPublicAccess/noLimitations"));

        assertStatementPresent(resource, DCTerms.title, reference);
        assertStatementPresent(resource, DCAT.keyword, reference);
        assertStatementPresent(resource, DCTerms.identifier, reference);
    }
}
