package de.landsh.opendata.csw2dcat;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

import java.util.regex.Pattern;

import static de.landsh.opendata.csw2dcat.TelephoneNumberClean.numberToURI;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertNull;

class TelephoneNumberCleanTest {
    static final Pattern tel = Pattern.compile("^tel:\\+(?:[0-9] ?){6,14}[0-9]$");

    @Test
    void testCleanTelephoneNumber() {
        assertEquals("tel:+494319880", numberToURI("tel:+494319880"));
        assertEquals("tel:+49-431-988-0", numberToURI("0431 988-0"));
        assertEquals("tel:+49-431-988-0", numberToURI("0431 988 0"));
        assertEquals("tel:+49-431-988-123", numberToURI("0431-988-123"));
        assertEquals("tel:+49-431-988-123", numberToURI("0431/988-123"));
        assertEquals("tel:+49-431-9880", numberToURI("0431-9880"));
        assertEquals("tel:+49-4319880", numberToURI("04319880"));
        assertEquals("tel:+494319880", numberToURI("+494319880"));
        assertEquals("tel:+494319880", numberToURI("+49(0)4319880"));
        assertEquals("tel:+49-4347-704-0", numberToURI("+49 (0) 04347 704 0"));
        assertEquals("tel:+49-160-12-34-56", numberToURI("0160 12 34 56"));
        assertEquals("tel:+1-555-2323", numberToURI("+1 555-2323"));
        assertEquals("tel:+123-456-7890", numberToURI("+123 456 7890"));
        assertEquals("tel:+49123456789", numberToURI("+4900123456789"));
        assertEquals("tel:+1-123-456-7890", numberToURI("+1 123 456 7890"));
        assertEquals("tel:+1-123-456-7890", numberToURI("+1/123/456/7890"));
    }

    @Test
    void testCleanTelephoneNumber_empty() {
        assertNull(numberToURI(null));
        assertNull(numberToURI(""));
        assertNull(numberToURI(" "));
    }

    @Test
    void testCleanTelephoneNumber_invalid() {
        Assertions.assertThrows(IllegalArgumentException.class, () -> {
            numberToURI("foobar");
        });
    }

}